; Switch.s
; Runs on LM3S811
; Provide functions that initialize a GPIO as an input pin and 
; allow reading of a logic switch (negative logic switch on 811 kit).
; Use bit-banded I/O.
; Jonathan Valvano
; September 21, 2012

; This example accompanies the book
;  "Embedded Systems: Introduction to Arm Cortex M3 Microcontrollers",
;  ISBN: 978-1469998749, Jonathan Valvano, copyright (c) 2012
;  Section 4.2.2, Program 4.2, Figure 4.9

;Copyright 2012 by Jonathan W. Valvano, valvano@mail.utexas.edu
;   You may use, edit, run or distribute this file
;   as long as the above copyright notice remains
;THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
;OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
;MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
;VALVANO SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
;OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
;For more information about my classes, my research, and my books, see
;http://users.ece.utexas.edu/~valvano/

; uses switch and LED on EKK-LM3S811 kit
; negative logic switch connected to PC4 (external pullup resistor)
; positive logic LED connected to PC5 (220ohn resistor)

PC4                EQU 0x40006040
PC5                EQU 0x40006080
GPIO_PORTC_DIR_R   EQU 0x40006400
GPIO_PORTC_AFSEL_R EQU 0x40006420
GPIO_PORTC_DEN_R   EQU 0x4000651C
SYSCTL_RCGC2_R     EQU 0x400FE108
SYSCTL_RCGC2_GPIOC EQU 0x00000004   ; port C Clock Gating Control

        AREA    |.text|, CODE, READONLY, ALIGN=2
        THUMB
        EXPORT  Start
Start
    BL  Switch_Init                 ; initialize PC4 and make it input
    ; set direction register
    LDR R1, =GPIO_PORTC_DIR_R       ; R1 = &GPIO_PORTC_DIR_R
    LDR R0, [R1]                    ; R0 = [R1]
    ORR R0, R0, #0x20               ; R0 = R0|0x04 (make PC5 output)
    STR R0, [R1]                    ; [R1] = R0
    ; regular port function
    LDR R1, =GPIO_PORTC_AFSEL_R     ; R1 = &GPIO_PORTC_AFSEL_R
    LDR R0, [R1]                    ; R0 = [R1]
    BIC R0, R0, #0x20               ; disable alt funct on PC5
    STR R0, [R1]                    ; [R1] = R0
    ; enable digital port
    LDR R1, =GPIO_PORTC_DEN_R       ; R1 = &GPIO_PORTC_DEN_R
    LDR R0, [R1]                    ; R0 = [R1]
    ORR R0, R0, #0x20               ; enable digital I/O on PC5
    STR R0, [R1]                    ; [R1] = R0
    LDR R4, =PC5                    ; R4 = &PC5
    MOV R5, #0x20                   ; R5 = LED on
    MOV R6, #0x00                   ; R6 = LED off
loop
    BL  Switch_Input
    CMP R0, #0x10                   ; R0 == 0x10?
    BEQ ledon                       ; if so, turn LED on
ledoff
    STR R6, [R4]                    ; [R4] = R6 = 0x00
    B   loop
ledon
    STR R5, [R4]                    ; [R4] = R5 = 0x20
    B   loop

;------------Switch_Init------------
; Make PC4 an input and enable digital I/O.
; Input: none
; Output: none
; Modifies: R0, R1
Switch_Init
    ; activate clock for Port C
    LDR R1, =SYSCTL_RCGC2_R         ; R1 = &SYSCTL_RCGC2_R
    LDR R0, [R1]                    ; R0 = [R1]
    ORR R0, R0, #SYSCTL_RCGC2_GPIOC ; R0 = R0|SYSCTL_RCGC2_GPIOC
    STR R0, [R1]                    ; [R1] = R0
    NOP
    NOP                             ; allow time to finish activating
    ; set direction register
    LDR R1, =GPIO_PORTC_DIR_R       ; R1 = &GPIO_PORTC_DIR_R
    LDR R0, [R1]                    ; R0 = [R1]
    BIC R0, R0, #0x10               ; make PC4 input
    STR R0, [R1]                    ; [R1] = R0
    ; regular port function
    LDR R1, =GPIO_PORTC_AFSEL_R     ; R1 = &GPIO_PORTC_AFSEL_R
    LDR R0, [R1]                    ; R0 = [R1]
    BIC R0, R0, #0x10               ; disable alt funct on PC4
    STR R0, [R1]                    ; [R1] = R0
    ; enable digital port
    LDR R1, =GPIO_PORTC_DEN_R       ; R1 = &GPIO_PORTC_DEN_R
    LDR R0, [R1]                    ; R0 = [R1]
    ORR R0, R0, #0x10               ; enable digital I/O on PC4
    STR R0, [R1]                    ; [R1] = R0
    BX  LR                          ; return

;------------Switch_Input------------
; Return the status of the negative logic switch.
; Input: none
; Output: R0  0x00 if pressed; 0x10 if not pressed
; Modifies: R1
Switch_Input
    LDR R1, =PC4                    ; pointer to PC4
    LDR R0, [R1]                    ; R0 = [R1] (read PC4)
    BX  LR                          ; return

    ALIGN                           ; make sure the end of this section is aligned
    END                             ; end of file
